﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Windows;
using System.Windows.Media;
using System.Windows.Threading;

using Arction.Wpf.Charting;
using Arction.Wpf.Charting.EventMarkers;
using Arction.Wpf.Charting.Titles;

namespace MultipleAsyncChartWindows
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		private List<ChartWindow> m_chartContainers;

		public MainWindow()
		{
			m_chartContainers = new List<ChartWindow>();

			InitializeComponent();
		}

		private void CreateNewChartWindow()
		{
			Thread newWindowThread =
				new Thread(
					new ThreadStart(
						delegate
						{
							// Create a new ChartWindow instance from a thread. 
							// This way it is not owned by the main UI thread.
							ChartWindow cc = new ChartWindow();
							cc.Closed += new EventHandler(cc_Closed);
							cc.Show();

							// Add the chart window to the list so that we can access it 
							// later.
							m_chartContainers.Add(cc);

							// Call Dispatcher.Run method and the chart window will 
							// have its own dispatcher ("UI thread").
							Dispatcher.Run();		
						}
					)
				);

			newWindowThread.SetApartmentState(ApartmentState.STA);
			newWindowThread.Start();
		}

		private void cc_Closed(object sender, EventArgs e)
		{
			// Remove the chart window from the list.
			m_chartContainers.Remove(sender as ChartWindow);
		}

		private void buttonOpen_Click(object sender, RoutedEventArgs e)
		{
			CreateNewChartWindow();
		}

		private void buttonSleep_Click(object sender, RoutedEventArgs e)
		{
			Thread.Sleep(5000);
		}

		private void buttonMarker_Click(object sender, RoutedEventArgs e)
		{
			// Let's use the same timestamp for every chart.
			DateTime now = DateTime.Now;

			foreach (ChartWindow chartContainer in m_chartContainers)
			{
				// Let's use ThreadPool to insert a marker to each chart. 
				// This way the "main window" stays resposive if one of the 
				// chart windows is doing something that blocks its 
				// dispatcher (e.g. Thread.Sleep is called). Marker is 
				// inserted when as soon as it's possible.

				ThreadPool.QueueUserWorkItem(
					new WaitCallback(
						delegate (Object state)
						{
							ChartWindow cc = state as ChartWindow;							
							cc.Dispatcher.Invoke(
								new Action(
									delegate
									{
										LightningChartUltimate chart = cc.Chart;
                                        chart.BeginUpdate(); 

                                        SeriesEventMarker marker = new SeriesEventMarker();
                                        marker.VerticalPosition = SeriesEventMarkerVerticalPosition.YAxisCenter;
                                        marker.XValue = chart.ViewXY.XAxes[0].DateTimeToAxisValue(now);
                                        marker.HorizontalPosition = SeriesEventMarkerHorizontalPosition.AtXValue;
                                        marker.Label.Text = "Marker";

                                        chart.ViewXY.PointLineSeries[0].SeriesEventMarkers.Add(marker);
                                        chart.EndUpdate();
									}
								)
							);
						}
					),
					chartContainer as Object
				);
			}
		}
	}
}
